﻿// Code for the Slider class, slider4 Example.

class Slider extends MovieClip
{
	// The slider consists of a thumb var and a track.
	private var _mThumbBar:MovieClip;
	private var _mTrack:MovieClip;
	
	// _nValue is the current value of the slider.
	private var _nValue:Number;
	
	// The listener is used to determine when the slider moves.
	private var _oMouseListener:Object;
	
	// These variables determine the height and width of the slider.
	private var _nWidth:Number;
	private var _nHeight:Number;
	
	// Declare _mComponentBox so compiler doesn't throw an error.
	private var _mComponentBox:MovieClip;
	
	public var addEventListener:Function;
	public var removeEventListener:Function;
	private var dispatchEvent:Function;
	
	// Define a getter so that _nValue is read only.
	public function get value( ):Number
	{
		return _nValue;
	}
	// Make this property inspectable
	[Inspectable(defaultValue=50)]
	public function set value(nValue:Number):Void
	{
		// Value must be between 0 and 100.
		if(nValue > 100)
		{
			nValue = 100;
		}
		else if(nValue < 0)
		{
			nValue = 0;
		}
		_nValue = nValue;
		
		// Update x-coordinate of thumbbar.
		_mThumbBar._x = _nValue / 100 * 
			(_nWidth + 0.5 - _mThumbBar._width);
	}
	
	// The constructor for the Slider.
	public function Slider( )
	{
		// Define default width and height.
		//_nWidth = 200;
		//_nHeight = 10;
		
		_nWidth = _width;
		_nHeight = _height;
		_xscale = 100;
		_yscale = 100;
		// Don't render control at runtime.
		_mComponentBox._width = 0;
		_mComponentBox._height = 0;
		_mComponentBox._visible = false;
		
		
		mx.events.EventDispatcher.initialize(this);
		
		// Initialize default value to 0.
		if(_nValue == undefined)
		{
			_nValue = 0;
		}
	
		// Create movie clips for the thumb bar and track.
		_mTrack = createEmptyMovieClip("_mTrack", 
									   getNextHighestDepth( ));
		_mThumbBar = createEmptyMovieClip("_mThumbBar", 
										  getNextHighestDepth( ));
	
		// Draw the slider
		draw( );
	
		// The listener object detects mouse moves and updates the slider.
		_oMouseListener = new Object( );
		_oMouseListener.onMouseMove = mx.utils.Delegate.create(this, update);
	
		// Define onPress and onRelease methods for the mThumbBar moviclip.
		_mThumbBar.onPress = 
			mx.utils.Delegate.create(this, onPressThumbBar);
		_mThumbBar.onRelease = 
			mx.utils.Delegate.create(this, onReleaseThumbBar);
		_mThumbBar.onReleaseOutside = _mThumbBar.onRelease;
		
		// Set the value property, which causes _x of the thumbbar
		// to be updated.
		value = _nValue;
	}
	
	private function draw( ):Void
	{
		// Clear previously drawn content.
		_mThumbBar.clear( );
		_mTrack.clear( );
		
		// Thumbbar is 5 pixels wide with height _nHeight.
		_mThumbBar.lineStyle(0, 0x000000, 0);
		_mThumbBar.beginFill(0, 100);
		_mThumbBar.lineTo(5, 0);
		_mThumbBar.lineTo(5, _nHeight);
		_mThumbBar.lineTo(0, _nHeight);
		_mThumbBar.lineTo(0, 0);
		_mThumbBar.endFill( );
		
		// Track is 10 pixels tall with width _nWidth.
		_mTrack.lineStyle(0, 0x000000, 100);
		_mTrack.lineTo(_nWidth, 0);
		_mTrack.lineTo(_nWidth, 5);
		_mTrack.lineTo(0, 5);
		_mTrack.lineTo(0, 0);
		
		// Vertically center the track.
		_mTrack._y = _height / 2 - 2.5;
	}
	
	private function onPressThumbBar( ):Void
	{
		// When user presses mouse on thumbbar, make it draggable.
		_mThumbBar.startDrag(false, 0, 0, 
			_mTrack._width - _mThumbBar._width + 0.5, 0);
		
		// Start detecting mouse move events.
		Mouse.addListener(_oMouseListener);
	}
	
	private function onReleaseThumbBar( ):Void
	{
		// When user releases thumbbar, stop dragging.
		_mThumbBar.stopDrag( );
		
		// Stop detecting mouse move events.
		Mouse.removeListener(_oMouseListener);
		update( );
	}
		
	private function update( ):Void
	{
		_nValue = Math.round(100 * (_mThumbBar._x ) / 
			(_mTrack._width + 0.5 - _mThumbBar._width));
		dispatchEvent({type: "change", target: this});
	}
	
	public function setSize(nWidth:Number, nHeight:Number):Void
	{
		_nWidth = nWidth;
		_nHeight = nHeight;
		draw( );
	}
}